@@ -68,10 +68,8 @@
 #include "Pass.hxx"
 
 using std::map;
-using std::string;
 using namespace simgear;
 
-
 ////////////////////////////////////////////////////////////////////////
 // Constructors and destructor.
 ////////////////////////////////////////////////////////////////////////
@@ -82,8 +80,10 @@ SGMaterial::_internal_state::_internal_state(Effect *e, bool l,
 {
 }
 
-SGMaterial::_internal_state::_internal_state(Effect *e, const string &t, bool l, 
-                                             const SGReaderWriterOptions* o)
+SGMaterial::_internal_state::_internal_state( Effect *e,
+                                              const std::string &t,
+                                              bool l,
+                                              const SGReaderWriterOptions* o )
     : effect(e), effect_realized(l), options(o)
 {
     texture_paths.push_back(std::make_pair(t,0));
@@ -96,19 +96,27 @@ void SGMaterial::_internal_state::add_texture(const std::string &t, int i)
 
 SGMaterial::SGMaterial( const SGReaderWriterOptions* options,
                         const SGPropertyNode *props,
-                        SGPropertyNode *prop_root )
+                        SGPropertyNode *prop_root,
+                        AreaList *a,
+						SGSharedPtr<const SGCondition> c)
 {
     init();
+    areas = a;
+    condition = c;
     read_properties( options, props, prop_root );
     buildEffectProperties(options);
 }
 
 SGMaterial::SGMaterial( const osgDB::Options* options,
                         const SGPropertyNode *props, 
-                        SGPropertyNode *prop_root)
+                        SGPropertyNode *prop_root,
+                        AreaList *a,
+                        SGSharedPtr<const SGCondition> c)
 {
     osg::ref_ptr<SGReaderWriterOptions> opt;
     opt = SGReaderWriterOptions::copyOrCreate(options);
+    areas = a;
+    condition = c;
     init();
     read_properties(opt.get(), props, prop_root);
     buildEffectProperties(opt.get());
@@ -132,23 +140,18 @@ SGMaterial::read_properties(const SGReaderWriterOptions* options,
     std::vector<SGPropertyNode_ptr> textures = props->getChildren("texture");
     for (unsigned int i = 0; i < textures.size(); i++)
     {
-        string tname = textures[i]->getStringValue();
+        std::string tname = textures[i]->getStringValue();
 
         if (tname.empty()) {
             tname = "unknown.rgb";
         }
 
-        SGPath tpath("Textures.high");
+        SGPath tpath("Textures");
         tpath.append(tname);
-        string fullTexPath = SGModelLib::findDataFile(tpath.str(), options);
+        std::string fullTexPath = SGModelLib::findDataFile(tpath.str(), options);
         if (fullTexPath.empty()) {
-            tpath.set("Textures");
-            tpath.append(tname);
-            fullTexPath = SGModelLib::findDataFile(tpath.str(), options);
-            if (fullTexPath.empty()) {
-                SG_LOG(SG_GENERAL, SG_ALERT, "Cannot find texture \""
-                        << tname << "\" in Textures or Textures.high folders.");
-            }
+            SG_LOG(SG_GENERAL, SG_ALERT, "Cannot find texture \""
+                    << tname << "\" in Textures folders.");
         }
 
         if (tpath.lower_extension() == "dds") {
@@ -170,22 +173,17 @@ SGMaterial::read_properties(const SGReaderWriterOptions* options,
         std::vector<SGPropertyNode_ptr> textures = texturesets[i]->getChildren("texture");
         for (unsigned int j = 0; j < textures.size(); j++)
         {
-            string tname = textures[j]->getStringValue();
+            std::string tname = textures[j]->getStringValue();
             if (tname.empty()) {
                 tname = "unknown.rgb";
             }
 
-            SGPath tpath("Textures.high");
+            SGPath tpath("Textures");
             tpath.append(tname);
-            string fullTexPath = SGModelLib::findDataFile(tpath.str(), options);
+            std::string fullTexPath = SGModelLib::findDataFile(tpath.str(), options);
             if (fullTexPath.empty()) {
-                tpath.set("Textures");
-                tpath.append(tname);
-                fullTexPath = SGModelLib::findDataFile(tpath.str(), options);
-                if (fullTexPath.empty() ) {
-                    SG_LOG(SG_GENERAL, SG_ALERT, "Cannot find texture \""
-                            << tname << "\" in Textures or Textures.high folders.");
-                }
+                SG_LOG(SG_GENERAL, SG_ALERT, "Cannot find texture \""
+                        << tname << "\" in Textures folders.");
             }
 
             if (j == 0) {
@@ -215,22 +213,16 @@ SGMaterial::read_properties(const SGReaderWriterOptions* options,
     std::vector<SGPropertyNode_ptr> masks = props->getChildren("object-mask");
     for (unsigned int i = 0; i < masks.size(); i++)
     {
-        string omname = masks[i]->getStringValue();
+        std::string omname = masks[i]->getStringValue();
 
         if (! omname.empty()) {
-            SGPath ompath("Textures.high");
+            SGPath ompath("Textures");
             ompath.append(omname);
-            string fullMaskPath = SGModelLib::findDataFile(ompath.str(), options);
-
-            if (fullMaskPath.empty()) {
-                ompath.set("Textures");
-                ompath.append(omname);
-                fullMaskPath = SGModelLib::findDataFile(ompath.str(), options);
-            }
+            std::string fullMaskPath = SGModelLib::findDataFile(ompath.str(), options);
 
             if (fullMaskPath.empty()) {
                 SG_LOG(SG_GENERAL, SG_ALERT, "Cannot find texture \""
-                        << omname << "\" in Textures or Textures.high folders.");
+                        << omname << "\" in Textures folders.");
             }
             else
             {
@@ -279,7 +271,8 @@ SGMaterial::read_properties(const SGReaderWriterOptions* options,
     building_coverage = props->getDoubleValue("building-coverage", 0.0);
     building_spacing = props->getDoubleValue("building-spacing-m", 5.0);
     
-    string bt = props->getStringValue("building-texture", "Textures/buildings.png");
+    std::string bt = props->getStringValue( "building-texture",
+                                            "Textures/buildings.png" );
     building_texture = SGModelLib::findDataFile(bt, options);    
     
     if (building_texture.empty()) {
@@ -340,17 +333,16 @@ SGMaterial::read_properties(const SGReaderWriterOptions* options,
     const SGPropertyNode* treeTexNode = props->getChild("tree-texture");
     
     if (treeTexNode) {
-        string treeTexPath = props->getStringValue("tree-texture");
+        std::string treeTexPath = props->getStringValue("tree-texture");
 
         if (! treeTexPath.empty()) {
-            SGPath treePath("Textures.high");
+            SGPath treePath("Textures");
             treePath.append(treeTexPath);
             tree_texture = SGModelLib::findDataFile(treePath.str(), options);
 
             if (tree_texture.empty()) {
-                treePath.set("Textures");
-                treePath.append(treeTexPath);
-                tree_texture = SGModelLib::findDataFile(treePath.str(), options);
+                SG_LOG(SG_GENERAL, SG_ALERT, "Cannot find texture \""
+                        << treeTexPath << "\" in Textures folders.");
             }
         }
     }
@@ -407,12 +399,6 @@ SGMaterial::read_properties(const SGReaderWriterOptions* options,
     } else {
         parameters = new SGPropertyNode();
     }
-
-    // Read conditions node
-    const SGPropertyNode *conditionNode = props->getChild("condition");
-    if (conditionNode) {
-        condition = sgReadCondition(prop_root, conditionNode);
-    }
 }
 
 
@@ -455,7 +441,7 @@ Effect* SGMaterial::get_effect(int i)
     return _status[i].effect.get();
 }
 
-Effect* SGMaterial::get_effect(const SGTexturedTriangleBin& triangleBin)
+Effect* SGMaterial::get_one_effect(int texIndex)
 {
     SGGuard<SGMutex> g(_lock);
     if (_status.empty()) {
@@ -463,7 +449,7 @@ Effect* SGMaterial::get_effect(const SGTexturedTriangleBin& triangleBin)
         return 0;
     }
     
-    int i = triangleBin.getTextureIndex() % _status.size();
+    int i = texIndex % _status.size();
     return get_effect(i);
 }
 
@@ -474,7 +460,7 @@ Effect* SGMaterial::get_effect()
 }
 
 
-osg::Texture2D* SGMaterial::get_object_mask(const SGTexturedTriangleBin& triangleBin)
+osg::Texture2D* SGMaterial::get_one_object_mask(int texIndex)
 {
     if (_status.empty()) {
         SG_LOG( SG_GENERAL, SG_WARN, "No mask available.");
@@ -483,7 +469,7 @@ osg::Texture2D* SGMaterial::get_object_mask(const SGTexturedTriangleBin& triangl
     
     // Note that the object mask is closely linked to the texture/effect
     // so we index based on the texture index, 
-    unsigned int i = triangleBin.getTextureIndex() % _status.size();
+    unsigned int i = texIndex % _status.size();
     if (i < _masks.size()) {
         return _masks[i].get();
     } else {
@@ -540,9 +526,9 @@ void SGMaterial::buildEffectProperties(const SGReaderWriterOptions* options)
     }
 }
 
-SGMaterialGlyph* SGMaterial::get_glyph (const string& name) const
+SGMaterialGlyph* SGMaterial::get_glyph (const std::string& name) const
 {
-    map<string, SGSharedPtr<SGMaterialGlyph> >::const_iterator it;
+    map<std::string, SGSharedPtr<SGMaterialGlyph> >::const_iterator it;
     it = glyphs.find(name);
     if (it == glyphs.end())
         return 0;
@@ -550,13 +536,43 @@ SGMaterialGlyph* SGMaterial::get_glyph (const string& name) const
     return it->second;
 }
 
-bool SGMaterial::valid() const
-{ 
-  if (condition) {
-    return condition->test();       
-  } else {
-    return true;
-  }
+bool SGMaterial::valid(SGVec2f loc) const
+{
+	SG_LOG( SG_TERRAIN, SG_BULK, "Checking materials for location ("
+			<< loc.x() << ","
+			<< loc.y() << ")");
+
+	// Check location first again the areas the material is valid for
+	AreaList::const_iterator i = areas->begin();
+
+	if (i == areas->end()) {
+		// No areas defined, so simply check against condition
+		if (condition) {
+			return condition->test();
+		} else {
+			return true;
+		}
+	}
+
+	for (; i != areas->end(); i++) {
+
+		SG_LOG( SG_TERRAIN, SG_BULK, "Checking area ("
+				<< i->x() << ","
+				<< i->y() << ") width:"
+				<< i->width() << " height:"
+				<< i->height());
+		// Areas defined, so check that the tile location falls within it
+		// before checking against condition
+		if (i->contains(loc.x(), loc.y())) {
+			if (condition) {
+				return condition->test();
+			} else {
+				return true;
+			}
+		}
+	}
+
+	return false;
 }
 
 
