@@ -262,7 +262,7 @@ ModelRegistry::readImage(const string& fileName,
         case GL_COMPRESSED_RED_GREEN_RGTC2_EXT:
         case GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT:
 
-            SG_LOG(SG_IO, SG_ALERT, "Image \"" << fileName << "\"\n"
+            SG_LOG(SG_IO, SG_WARN, "Image \"" << fileName << "\"\n"
                    "uses compressed textures which cannot be supported on "
                    "some systems.\n"
                    "Please decompress this texture for improved portability.");
