@@ -102,6 +102,7 @@ SGNewCloud::SGNewCloud(const SGPath &texture_root, const SGPropertyNode *cld_def
     max_shade_lighting_factor = cld_def->getFloatValue("max-shade-lighting-factor", min(min_shade_lighting_factor  + 0.1, 1.0));
     
     zscale = cld_def->getFloatValue("z-scale", 1.0);
+    alpha_factor = cld_def->getFloatValue("alpha-factor",1.0);
     texture = cld_def->getStringValue("texture", "cl_cumulus.png");
 
     // Create a new Effect for the texture, if required.
@@ -150,8 +151,10 @@ osg::ref_ptr<EffectGeode> SGNewCloud::genCloud() {
     float height = min_height + mt_rand(seed) * (max_height - min_height) - min_sprite_height;
     
     if (width  < 0.0) { width  = 0.0; }
-    if (height < 0.0) { height = 0.0; }
     
+    // Protect against divide by 0 issues later when assigning index_y
+    if (height <= 0.0) { height = 0.01; }
+
     // Determine appropriate shading factors
     float top_factor = min_top_lighting_factor + mt_rand(seed) * (max_top_lighting_factor - min_top_lighting_factor);
     float middle_factor = min_middle_lighting_factor + mt_rand(seed) * (max_middle_lighting_factor - min_middle_lighting_factor);
@@ -169,7 +172,8 @@ osg::ref_ptr<EffectGeode> SGNewCloud::genCloud() {
                                                       bottom_factor,
                                                       shade_factor,
                                                       height,
-                                                      zscale);
+                                                      zscale,
+                                                      alpha_factor);
         
     // Determine the cull distance. This is used to remove sprites that are too close together.
     // The value is squared as we use vector calculations.
