@@ -44,7 +44,7 @@ SGBinding::SGBinding(const SGPropertyNode* node, SGPropertyNode* root)
 SGBinding::~SGBinding()
 {
   if(_arg && _arg->getParent())
-    _arg->getParent()->removeChild(_arg->getName(), _arg->getIndex(), false);
+    _arg->getParent()->removeChild(_arg->getName(), _arg->getIndex());
 }
 
 void
